/**
 * AuroraView Browser Extension Integration
 *
 * This module provides utilities for integrating Chrome/Firefox extensions
 * with AuroraView Python applications.
 *
 * @module @auroraview/sdk/browser-extension
 */

export {
  AuroraViewClient,
  createAuroraViewClient,
  getDefaultClient,
  type AuroraViewClientConfig,
  type CallOptions,
} from "./client";
