"""AuroraView MCP Server - AI-powered interaction with WebView applications."""

from auroraview_mcp.server import create_server, mcp

__version__ = "0.1.0"
__all__ = ["create_server", "mcp", "__version__"]
